<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://makewebbetter.com/
 * @since      1.0.0
 *
 * @package    iconwoo-integration
 * @subpackage iconwoo-integration/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    iconwoo-integration
 * @subpackage iconwoo-integration/admin
 * @author     MakeWebBetter <webmaster@makewebbetter.com>
 */
class Icontact_Woo_Integration_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

		// let's modularize our codebase, all the admin actions in one function. 
		$this->admin_action();
	}
	/**
	 * all admin actions.
	 * 
	 * @since 1.0.0
	 */
	public function admin_action(){
		// add submenu icontact in woocommerce top menu.
		add_action('admin_menu', array($this,'add_icontactwoo_submenu'));
	}
	/**
	 * add icontact submenu in woocommerce menu.
	 *
	 * @since 1.0.0
	 */
	public function add_icontactwoo_submenu(){

		add_submenu_page('woocommerce',__('iContactPRO','icontact'), __('iContactPRO','icontact'), 'manage_woocommerce', 'icontactwoo', array(&$this,'icontact_configurations'));
	}
	/**
	 * all the configuration related fields and settings.
	 * 
	 * @return html  all the settings and configuration options for icontact.
	 * @since 1.0.0
	 */
	public function icontact_configurations(){
		global $icontactwoo;
		

		include_once ICONTACT_WOO_ABSPATH . 'admin/templates/icontact-woo-main-template.php';
		
	}
	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		$screen = get_current_screen();
      	if( isset( $screen->id ) && $screen->id == 'woocommerce_page_icontactwoo' ) {

			wp_enqueue_style( $this->plugin_name."-bootstrap-style", plugin_dir_url( __FILE__ ) . 'css/bootstrap.css', array(), $this->version, 'all' );

			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/icontact-woo-integration-admin.css', array(), $this->version, 'all' );

			wp_register_style( 'woocommerce_admin_styles', WC()->plugin_url() . '/assets/css/admin.css', array(), WC_VERSION );

			wp_enqueue_style( 'woocommerce_admin_menu_styles' );
					
			wp_enqueue_style( 'woocommerce_admin_styles' );

			wp_enqueue_style( 'thickbox' );
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		$screen = get_current_screen();
      	if( isset( $screen->id ) && $screen->id == 'woocommerce_page_icontactwoo' ) {


			wp_register_script( 'woocommerce_admin', WC()->plugin_url() . '/assets/js/admin/woocommerce_admin.js', array( 'jquery', 'jquery-blockui', 'jquery-ui-sortable', 'jquery-ui-widget', 'jquery-ui-core', 'jquery-tiptip', 'wc-enhanced-select' ), WC_VERSION );

			wp_register_script( 'icontact_woo_script', plugin_dir_url( __FILE__ ) . 'js/icontact-woo-integration-admin.js', array( 'jquery' ), $this->version, true );
			$locale  = localeconv();
			$decimal = isset( $locale['decimal_point'] ) ? $locale['decimal_point'] : '.';
			$params = array(
				'i18n_decimal_error'                => sprintf( __( 'Please enter in decimal (%s) format without thousand separators.', "iconwoo" ), $decimal ),
				'i18n_mon_decimal_error'            => sprintf( __( 'Please enter in monetary decimal (%s) format without thousand separators and currency symbols.', 'iconwoo' ), wc_get_price_decimal_separator() ),
				'i18n_country_iso_error'            => __( 'Please enter in country code with two capital letters.', 'iconwoo' ),
				'i18_sale_less_than_regular_error'  => __( 'Please enter in a value less than the regular price.', 'iconwoo' ),
				'decimal_point'                     => $decimal,
				'mon_decimal_point'                 => wc_get_price_decimal_separator(),
				'strings' => array(
					'import_products' => __( 'Import', 'iconwoo' ),
					'export_products' => __( 'Export', 'iconwoo' ),
				),
				'urls' => array(
					'import_products' => esc_url_raw( admin_url( 'edit.php?post_type=product&page=product_importer' ) ),
					'export_products' => esc_url_raw( admin_url( 'edit.php?post_type=product&page=product_exporter' ) ),
				),
			);
			wp_enqueue_script( 'jquery-ui-datepicker' );
			wp_localize_script( 'woocommerce_admin', 'woocommerce_admin', $params );
			wp_enqueue_script( 'woocommerce_admin' );
			wp_localize_script('icontact_woo_script',
					'iconwooi18n', array( 'ajaxUrl' => admin_url('admin-ajax.php'),
					'iconwooSecurity' => wp_create_nonce( 'iconwoo_security'), 
					'iconwooWentWrong' => __( 'Something went wrong, please try again later!', 'iconwoo' ), 
					'iconwooSuccess' => __( 'Setup is completed successfully!', 'iconwoo' ),
					'iconwooCreatingProperty' => __( 'Field created successfully', 'iconwoo' ),
					'iconwooSetupCompleted' => __('Setup completed!', 'iconwoo'),
					'iconwooConnectTab' => admin_url() . 'admin.php?page=icontactwoo&iconwoo_tab=icontact_woo_connect',
					'iconwooCustomFields' => admin_url() . 'admin.php?page=icontactwoo&iconwoo_tab=icontact_woo_custom_contact_field',
					'iconwooNoFieldsFound' => __( 'It seems that no fields are selected. Please select atleast one field for the setup to start','iconwoo'),
				)
			);

			wp_enqueue_script('icontact_woo_script');
			wp_enqueue_script('thickbox');

			wp_register_script( $this->plugin_name."-bootstap-script", plugin_dir_url( __FILE__ ) . 'js/bootstrap.js', array( 'jquery' ), $this->version, false );

			wp_enqueue_script( $this->plugin_name."-bootstap-script" );
		}

	}


	/**
	 * Update schedule data with custom time.
	 *
	 * @since    1.0.0
	 * @param      string    $schedules       Schedule data.
	 */
	public function icontact_woo_set_cron_time( $schedules ) {

		if( !isset( $schedules[ "icontact-woo-5min-cron" ] ) ) {

	        $schedules["icontact-woo-5min-cron"] = array(
	            'interval' => 5*60,
	            'display' => __( 'Once every 5 minutes', 'iconwoo' )
	        );
	    }

	    return $schedules;
	}

	/**
	 * Schedule Executes when user data is update.
	 *
	 * @since    1.0.0
	 * @param      string    $schedules       Schedule data.
	 */

	public function icontact_woo_cron_schedule(){

		$plugin_enable = get_option('icontact_woo_settings_enable','no');

		if ($plugin_enable == 'yes') {
			
			$icontact_woo_user_choice = Icontact_Woo_Integration::icontact_woo_user_choice();

			$args['meta_query'] = array(
				array(
					'key'		=> 'icontact_woo_user_data_change',
					'value'		=> 'yes',
				 	'compare'	=> '=='
				 	)
				);


			$updated_users = get_users( $args );

			$icontact_woo_users = apply_filters('icontact_woo_users', $updated_users );

			$icontact_woo_unique_users = array();

			if (is_array($icontact_woo_users) && count( $icontact_woo_users)) {
				
				foreach( $icontact_woo_users as $key => $value ) {

					if( in_array( $value->ID, $icontact_woo_unique_users ) ) {

						continue;
					}
					else {

						$icontact_woo_unique_users[]= $value->ID;
					}
				}
			}

			if (isset($icontact_woo_unique_users) && $icontact_woo_unique_users != null && count( $icontact_woo_unique_users )) 
			{

				foreach ($icontact_woo_unique_users as $key => $ID) {

					$icontact_woo_customer = new IcontactWooCustomer($ID);

					$properties = $icontact_woo_customer->get_contact_properties();
					           						
					$filtered_properties = array();

					if ($icontact_woo_user_choice == 'yes') {

						$selected_properties = Icontact_Woo_Integration::icontact_woo_user_selected_fields();

						if( is_array( $selected_properties ) && count( $selected_properties ) && is_array( $properties ) && count( $properties ) )
						{

							foreach( $properties as $field => $single_property ) {

								if( in_array( $field, $selected_properties ) ) {

									$filtered_properties[$field] = $single_property;
								}
							}
						}
					}
					else{

						$filtered_properties = $properties;
					}
					
					
					$fname 		= get_user_meta( $ID, 'first_name', true );
					$lname 		= get_user_meta( $ID, 'last_name', true );
					$company 	= get_user_meta( $ID, 'billing_company', true );
					$phone 		= get_user_meta( $ID, 'billing_phone', true );
					$email_id 	= $icontact_woo_customer->get_email();

					$filtered_properties['firstName'] 	= $fname;
					$filtered_properties['lastName'] 	= $lname;
					$filtered_properties['phone'] 		= $phone;
					$filtered_properties['email'] 		= $email_id;
					
					$properties = apply_filters( 'icontact_woo_map_new_properties', $filtered_properties, $ID );
					
					$app_id 		= Icontact_Woo_Integration::get_app_id();
					$app_username 	= Icontact_Woo_Integration::get_api_username();
					$app_password 	= Icontact_Woo_Integration::get_app_password();
					$account_id 	= get_option('account_id_icontact','');
					$folder_id 		= get_option('folder_id_icontact','');
					
					$headers 		= IcontactWooConnectionManager::get_instance()->set_headers($app_id, $app_username, $app_password);

					$check_account 	= IcontactWooConnectionManager::get_instance()->get_contact_info($app_id, $app_username, $app_password, $account_id, $folder_id,$headers, $email_id);

					$check_account 	= json_decode($check_account);
					
					if (empty($check_account->contacts)) {

						//create account
						$contact = IcontactWooConnectionManager::get_instance()->create_contact($app_id, $app_username, $app_password, $account_id, $folder_id, $headers,  $filtered_properties);

						$contact_id = json_decode($contact)->contacts[0]->contactId;

						$list_id = get_option('icontact_mwb_global_list_id');
						
						IcontactWooConnectionManager::get_instance()->add_contact_to_list($app_id, $app_username, $app_password, $account_id, $folder_id, $headers, $contact_id, $list_id);	
					}
					else{

						//update account
						$a =IcontactWooConnectionManager::get_instance()->update_contact($app_id, $app_username, $app_password, $account_id, $folder_id, $headers, $filtered_properties, $email_id);
						
					}
					update_user_meta( $ID, 'icontact_woo_user_data_change', 'no' );
				}
			}

		}
		
	}

	/**
	 * updating users list to be updated on iContact on order status transition
	 * 
	 * @since 1.0.0
	 */

	public function icontact_woo_update_changes( $order_id ){

		if( !empty( $order_id ) ) {

			$user_id = (int)get_post_meta( $order_id, '_customer_user', true );

			if( $user_id != 0 && $user_id > 0 ) {

				update_user_meta( $user_id, 'icontact_woo_user_data_change', 'yes' );
			}
		}
	}

	/**
	 * General setting after setup of plugin.
	 * 
	 * @return array  woocommerce_admin_fields acceptable fields in array.
	 * @since 1.0.0
	 */
	public static function icontact_woo_settings() {

		$basic_settings = array();

		$basic_settings[] = array(
			'title' => __('Plugin related settings', 'iconwoo'),  
			'id'	=> 'icontact_woo_settings_title', 
			'type'	=> 'title'	
		);

		$basic_settings[] = array(
			'title' => __('Enable/Disable', 'iconwoo'),
			'id'	=> 'icontact_woo_settings_enable', 
			'desc'	=> __('Turn on/off the integration', 'iconwoo'),
			'type'	=> 'checkbox'
		);

		$basic_settings[] = array(
			'title' => __('Enable/Disable', 'iconwoo'),
			'id'	=> 'icontact_woo_log', 
			'desc'	=> sprintf( __('Enable logging of the requests. You can view iContact log file from <a href="%s">Here</a>', 'iconwoo'), '?page=wc-status&tab=logs'),
			'type'	=> 'checkbox'
		);

		$basic_settings[] = array(
			'type' => 'sectionend',
	        'id' => 'iconwoo_settings_end'
		);

		return $basic_settings;
	}

	/**
	 * woocommerce privacy policy
	 * 
	 * @since 1.0.0
	 */

	public function icontact_woo_add_privacy_message() {

		if ( function_exists( 'wp_add_privacy_policy_content' ) ) {

			$content = '<p>' . __( 'We use your email to send your Orders related data over iContactPRO.','iconwoo' ) . '</p>';

			$content .='<p>' . __( 'iContactPRO is an inbound marketing and sales platform that helps companies attract visitors, convert leads, and close customers.', 'iconwoo' ) . '</p>';

			$content .= '<p>' .  __( 'Please see the ', 'iconwoo' ) . '<a href="https://www.icontact.com/gdpr" target="_blank" >' . __( 'iContact Data Privacy', 'iconwoo' ) . '</a>' .  __( ' for more details.', 'iconwoo' ) . '</p>';

			if ( $content ) {

				wp_add_privacy_policy_content( __( 'Enhanced iContactPro WooCommerce Integration', 'iconwoo' ), $content );
			}
		}
	}
}
