<?php 
/**
 * Manage all contact properties.
 *
 * @link       https://makewebbetter.com/
 * @since      1.0.0
 *
 * @package    iconwoo-integration
 * @subpackage iconwoo-integration/includes
 */

/**
 * Manage all contact properties.
 *
 * Provide a list of functions to manage all the information
 * about contacts properties and lists along with option to 
 * change/update the mapping field on icontact.
 *
 * @package    iconwoo-integration
 * @subpackage iconwoo-integration/includes
 * @author     MakeWebBetticonwoo-integrationer <webmaster@makewebbetter.com>
 */

class IcontactWooContactProperties{

	/**
	 * Contact Property Groups.
	 *
	 * @access private
	 * @since 1.0.0
	 */
	private $groups;

	/**
	 * Contact Properties.
	 *
	 * @access private
	 * @since 1.0.0
	 */
	private $properties;

	/**
	 * IcontactWooContactProperties Instance.
	 *
	 * @access protected
	 * @since 1.0.0
	 */
	protected static $_instance = null;

	/**
	 * Main IcontactWooContactProperties Instance.
	 *
	 * Ensures only one instance of IcontactWooContactProperties is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @static
	 * @return IcontactWooContactProperties - Main instance.
	 */
	public static function get_instance() {

		if ( is_null( self::$_instance ) ) {

			self::$_instance = new self();
		}

		return self::$_instance;
	}

	/**
	 * Define the contact prooperties related functionality.
	 *
	 * Set the contact groups and properties that we are going to use
	 * for creating/updating the contact information for our tacking purpose
	 * and providing other developers to add there field and group for tracking
	 * too by simply using our hooks.
	 *
	 * @access public
	 * @since    1.0.0
	 */
	public function __construct(){

		$this->groups = $this->_set( 'groups' );
		$this->properties = $this->_set( 'properties' );
	}

	/**
	 * get groups/properties.
	 * 
	 * @param  string 		  groups/properties
	 * @return array          Array of groups/properties information.
	 */
	public function _get( $option, $groupName="" ){

		if( $option == "groups" ) {

			return $this->groups;
		}
		elseif( $option == "properties" ) {
			
			if( !empty( $groupName ) && isset( $this->properties[ $groupName ] )  ){

				return $this->properties[ $groupName ];
			}

			return $this->properties;
		}
	}

	/**
	 * get an array of required option.
	 * 
	 * @param  String   	$option  		the identifier.
	 * @return Array  		An array of values.	
	 * @since 1.0.0		     
	 */
	private function _set( $option ){

		$values = array();

		//if we are looking for groups, let us add our predefined groups.
		if( $option == 'groups' ) {

			// customer details
			$values[] = array( 'name' => 'customer_group', 'displayName' => __( 'Customer Group', 'iconwoo' ) );
			// order details.
			$values[] = array( 'name' => 'order', 'displayName' => __( 'Order', 'iconwoo' ) );
			// RFM details.
			$values[] = array( 'name' => 'rfm_fields', 'displayName' => __( 'RFM Information', 'iconwoo' ) );
			// Lifecycle Stage details.
			$values[] = array( 'name' => 'lifecyle_stage', 'displayName' => __( 'Lifecycle Stage', 'iconwoo' ) );
		}
		// if we are looking for properties.
		elseif( $option == 'properties' ) {

			$values = $this->get_all_active_groups_properties();
		}
		// add your values to the either groups or properties or segments.
		return apply_filters( 'icontact_woo_contact_'.$option, $values );
	}

	/**
	 * check for the active groups and get there properties.
	 * 
	 * @return Array Properties array with there associated group.
	 * @since 1.0.0
	 */
	private function get_all_active_groups_properties() {

		$active_groups_properties = array();
		//get all the active groups.
		$active_groups = $this->get_active_groups();

		//check if we get active groups in the form of array, and has groups.
		if( is_array( $active_groups ) && count( $active_groups ) ){

			foreach( $active_groups as $active_group ){

				if( !empty( $active_group ) && !is_array( $active_group ) ){

					$active_groups_properties[ $active_group ] = $this->_get_group_properties( $active_group );
				}
			}
		}
		// add your active group properties if you want.
		return apply_filters( 'icontact_woo_active_groups_properties', $active_groups_properties );
	} 

	/**
	 * Filter for active groups only.
	 * 
	 * @return Array active group names.
	 * @since 1.0.0
	 */
	private function get_active_groups(){

		$active_groups = array();

		$all_groups = $this->_get( 'groups' );

		if( is_array( $all_groups ) && count( $all_groups ) ){

			foreach( $all_groups as $group_details ){

				$group_name = isset( $group_details[ 'name' ] ) ? $group_details[ 'name' ] : '';

				if( !empty( $group_name ) ){

					$is_active = get_option( 'icontact_woo_active_group'.$group_name, true );

					if( $is_active ){

						$active_groups[] = $group_name;
					}
				}
			}
		}
		// let's developer manage there groups seperately if they want.
		return apply_filters( 'icontact_woo_active_groups', $active_groups );
	}

	/**
	 * get all the groups properties.
	 * 
	 * @param   string     $group_name     name of the existed valid icontact contact properties group.
	 * @return  Array      Properties array.
	 * @since 1.0.0
	 */
	private function _get_group_properties( $group_name ){

		$group_properties = array();
		//if the name is not empty.
		if( !empty( $group_name ) ){

			if( $group_name == "customer_group" ) {

				$group_properties[] = array(
					"alias" 	=> "mwb_customer_group",
					"label" 	=> __( 'Customer Group/ User role', 'iconwoo' ),
					"type" 		=> "text",
					"isVisible" => 0
				);
			}
			elseif( $group_name == "order" ) {

				$group_properties[] = array(
					"alias" 	=> "mwb_last_order_stat",
					"label" 	=> __( 'Last Order Status', 'iconwoo' ),
					"isVisible" => 0,
					"type" 		=> "text"
				);		
				
				$group_properties[] = array(
					"alias" 	=> "mwb_last_order_ff_stat",
					"label" 	=> __( 'Last Order Fulfillment Status', 'iconwoo' ),
					"isVisible" => 0,
					"type" 		=> "text"
				);		
				
				$group_properties[] = array(
					"alias" 	=> "mwb_last_order_track_num",
					"isVisible" => 0,
					"label"		=> __( 'Last Order Tracking Number', 'iconwoo' ),
					"type" 		=> "number", 
				);
				
				$group_properties[] = array(
					"alias" 	=> "mwb_last_order_ship_date",
					"isVisible" => 0,
					"label" 	=> __( 'Last Order Shipment Date', 'iconwoo' ),
					"type" 		=> "date",   
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_last_order_num",
					"isVisible" => 0,
					"label" 	=> __( 'Last Order Number', 'iconwoo' ),
					"type" 		=> "number",
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_current_orders",
					"isVisible" => 0,
					"label" 	=> __( 'Total Number of Current Orders', 'iconwoo' ),
					"type"		=> "number",
				);
			}
			elseif( $group_name == "rfm_fields" ) {

				$group_properties[] = array(
					"alias" 	=> "mwb_total_val_of_orders",
					"isVisible" => 0,
					"label" 	=> __( 'Total Value of Orders', 'iconwoo' ),
					"type" 		=> "number",
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_avg_order_value",
					"isVisible" => 0,
					"label" 	=> __( 'Average Order Value', 'iconwoo' ),
					"type" 		=> "number",   
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_total_orders",
					"isVisible" => 0,
					"label" 	=> __( 'Total Number of Orders', 'iconwoo' ),
					"type"		=> "number",  
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_first_order_val",
					"isVisible" => 0,
					"label" 	=> __( 'First Order Value', 'iconwoo' ),
					"type" 		=> "number",  
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_first_order_date",
					"isVisible" => 0,
					"label" 	=> __( 'First Order Date', 'iconwoo' ),
					"type" 		=> "date",
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_last_order_val",
					"isVisible" => 0,
					"label" 	=> __( 'Last Order Value', 'iconwoo' ),
					"type" 		=> "number",  
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_last_order_date",
					"isVisible" => 0,
					"label" 	=> __( 'Last Order Date', 'iconwoo' ),
					"type" 		=> "date",  
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_acc_creation_date",
					"isVisible" => 0,
					"label" 	=> __( 'Account Creation Date', 'iconwoo' ),
					"type" 		=> "date",
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_order_monetary",
					"isVisible" => 0,
					"label" 	=> __( 'Order Monetary Rating', 'iconwoo' ),
					"type" 		=> "number"
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_order_frequency",
					"isVisible" => 0,
					"label" 	=> __( 'Order Frequency Rating', 'iconwoo' ),
					"type" 		=> "number"
				);

				$group_properties[] = array(
					"alias" 	=> "mwb_order_recency",
					"isVisible" => 0,
					"label" 	=> __( 'Order Recency Rating', 'iconwoo' ),
					"type" 		=> "number"
				);

			}
			elseif ($group_name == "lifecyle_stage"){

				$group_properties[] = array(
					"alias" 	=> "mwb_lifecycle_stage",
					"isVisible" => 0,
					"label" 	=> __( 'Lifecycle', 'iconwoo' ),
					"type" 		=> "text"
				);

			}
		}

		return apply_filters( 'icontact_woo_group_properties', $group_properties, $group_name );
	}

	/**
	 * formatted options for user role enumaration.
	 * 
	 * @return JSON    formatted json encoded array of user role options.
	 * @since 1.0.0
	 */
	public static function get_user_roles(){

		$exiting_user_roles = array();
		
		if ( ! function_exists( 'get_editable_roles' ) ) {

		    require_once ABSPATH . 'wp-admin/includes/user.php';
		}
		
		global $wp_roles;

		$user_roles = !empty( $wp_roles->role_names )?$wp_roles->role_names:array();

		if( is_array( $user_roles ) && count( $user_roles ) ) {

			foreach ( $user_roles as $role => $role_info ) {

				$role_label = !empty( $role_info ) ? $role_info : $role;

				$exiting_user_roles[] = array( 'label' => $role_label, 'value' => $role ); 
			}
		}

		$exiting_user_roles[] 	= array( 'label' => 'Nil', 'value' => 'null' );
		$exiting_user_roles 	= apply_filters( 'icontact_woo_user_role_options', $exiting_user_roles );

		return $exiting_user_roles;
	}

	/**
	 * get all available woocommerce order statuses
	 * 
	 * @return JSON Order statuses in the form of enumaration options.
	 * @since 1.0.0
	 */
	public static function get_order_statuses(){

		$all_wc_statuses = array();

		//get all statuses
		$all_status = wc_get_order_statuses();
		
		//if status available
		if( is_array( $all_status ) && count( $all_status ) ){

			foreach( $all_status as $status_id => $status_label ){

				$all_wc_statuses[] = array( 'label' => $status_label, 'value' => $status_id );
			}
		}
		$all_wc_statuses[] = array( 'label' => 'Nil', 'value' => 'null' );
		$all_wc_statuses = apply_filters( 'icontact_woo_order_status_options', $all_wc_statuses );

		return $all_wc_statuses;
	}
}