<?php  
/**
 * All customer details.
 *
 * @link       https://makewebbetter.com/
 * @since      1.0.0
 *
 * @package    iconwoo-integration
 * @subpackage iconwoo-integration/includes
 */

/**
 * Stores all customer data that needs to be updated on icontact.
 *
 * Provide a list of properties and associated data for customer
 * so that at the time of updating a customer on icontact we can 
 * simply create an instance of this class and get everything 
 * managed.
 *
 * @package    iconwoo-integration
 * @subpackage iconwoo-integration/includes
 * @author     MakeWebBetter <webmaster@makewebbetter.com>
 */

class IcontactWooCustomer{

	/**
	 * contact in the form of acceptable by icontact.
	 *
	 * @since 1.0.0
	 * @access public
	 * @var json
	 */
	public $contact;

	/**
	 * WooCommerce Customer ID
	 *
	 * @since 1.0.0
	 * @access public
	 * @var json
	 */
	public $_contact_id;

	/**
	 * Contact Properties.
	 *
	 * @since 1.0.0
	 * @access private
	 * @var Array
	 */
	private $_properties = array();

	/**
	 * instance of IcontactWooPropertyCallbacks class
	 *
	 * @since 1.0.0
	 * @access private
	 * @var IcontactWooPropertyCallbacks
	 */
	private $_callback_instance = null;

	/**
	 * Load the modified customer properties.
	 *
	 * Set all the modified customer properties so that they will be
	 * ready in the form of directly acceptable by icontact api.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $contact_id ){

		//load the contat id in the class property.
		$this->_contact_id = $contact_id;

		//store the instance of property callback.
		$this->_callback_instance = new IcontactWooPropertyCallbacks( $this->_contact_id);

		//prepare the modified fields data and store it in the contact
		$this->prepare_modified_fields();
	}

	/**
	 * get user email.
	 * 
	 * @since 1.0.0
	 */

	public function get_email(){

		return $this->_callback_instance->_get_email();
	}

	/**
	 * contacts all properties.
	 * 
	 * @return array    and key value pair array of properties.
	 * @since 1.0.0
	 */

	public function get_contact_properties(){

		// let others decide if they have modified fields in there integration.
		$this->_properties = apply_filters( 'icontact_woo_contact_modified_fields', $this->_properties, $this->_contact_id );
		return $this->_properties;
	}

	/**
	 * Format modified fields of customer.
	 *
	 * Check for all the modified fields till the last update
	 * and prepare them in the icontact api acceptable form.
	 * 
	 * @since 1.0.0
	 * @access private
	 */

	public function prepare_modified_fields(){

		$modified_fields = IcontactWooContactProperties::get_instance()->_get('properties');

		//if some data are updated after last update with icontact.
		if (is_array($modified_fields) && count($modified_fields) ) {
			//loop them all as they are in form of groups and fields.
			foreach ($modified_fields as $group_fields) {
				//check if fields are there in group field.
				if (is_array($group_fields)) {
					//loop each field
					foreach ($group_fields as $field) {
						$property = $this->_prepare_property( $field['alias']);
						$this->_properties[ $field['alias']] = $property;
					}
				}
			}
		}
	}

	/**
	 * prepare property in the form of key value accepted by icontact.
	 * 
	 * @param  array    $property     array of the property details to validate the value.
	 * @return array               formatted key value pair.
	 */

	public function _prepare_property($property){

		//if property name is not empty
		if (!empty($property)) {
			
			$property_val = $this->_callback_instance->_get_property_value( $property, $this->_contact_id );

			return $property_val;
		}
	}

}