<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://makewebbetter.com/
 * @since      1.0.0
 *
 * @package    iconwoo-integration
 * @subpackage iconwoo-integration/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    iconwoo-integration
 * @subpackage iconwoo-integration/includes
 * @author     MakeWebBetter <webmaster@makewebbetter.com>
 */
class Icontact_Woo_Integration {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Icontact_Woo_Integration_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'ICONTACT_WOO_VERSION' ) ) {
			$this->version = ICONTACT_WOO_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'icontact-woo-integration';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Icontact_Woo_Integration_Loader. Orchestrates the hooks of the plugin.
	 * - Icontact_Woo_Integration_i18n. Defines internationalization functionality.
	 * - Icontact_Woo_Integration_Admin. Defines all hooks for the admin area.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-icontact-woo-integration-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-icontact-woo-integration-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-icontact-woo-integration-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-icontact-woo-integration-public.php';

		/**
		 * The class responsible for all api actions with icontact.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-icontact-woo-connection-manager.php';

		/**
		 * The class contains all the information related to customer groups and properties.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-icontact-woo-contact-properties.php';

		/**
		 * The class contains are readymade contact details to send it to 
		 * icontact.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-icontact-woo-customer.php';

		/**
		 * The class responsible for property values.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-icontact-woo-property-callbacks.php';
		/**
		 * The class responsible for handling ajax requests.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-icontact-woo-ajax-handler.php';


		$this->loader = new Icontact_Woo_Integration_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Icontact_Woo_Integration_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Icontact_Woo_Integration_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Icontact_Woo_Integration_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		$this->loader->add_filter( 'cron_schedules', $plugin_admin, 'icontact_woo_set_cron_time' );

		$this->loader->add_action( 'admin_init', $plugin_admin, 'icontact_woo_add_privacy_message' );

		if ($this->is_plugin_enable() == 'yes') {
			
			if( $this->is_setup_completed() ) {

				$this->loader->add_action( 'icontact_woo_cron_schedule', $plugin_admin, 'icontact_woo_cron_schedule' );
				$this->loader->add_action( 'woocommerce_order_status_changed', $plugin_admin, 'icontact_woo_update_changes' );
			}
		}
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Icontact_Woo_Integration_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'profile_update', $plugin_public, 'icontact_woo_save_account_details' );
		$this->loader->add_action( 'user_register', $plugin_public, 'icontact_woo_save_account_details' );
		$this->loader->add_action( 'woocommerce_checkout_update_user_meta', $plugin_public, 'icontact_woo_save_account_details' );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Icontact_Woo_Integration_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	/**
	 * predefined default iconwoo tabs.
	 * @return 	Array 		An key=>value pair of icontact tabs.
	 */
	public function icontact_woo_default_tabs(){

		$default_tabs = array();

		$default_tabs['icontact_woo_overview'] = array( "name" => __( 'Overview', 'iconwoo' ), "dependency" => "" );
		
		$default_tabs['icontact_woo_connect'] = array( "name" => __( 'Connect', 'iconwoo' ), "dependency" => array("icontact_woo_get_started") );

		$default_tabs['icontact_woo_custom_contact_field'] = array( "name" => __( 'Custom Fields', 'iconwoo' ), "dependency" => array("is_auth_success") );

		$common_dependency = array("is_auth_success","is_setup_completed");

		$default_tabs['icontact_woo_segments'] = array( "name" => __( 'Segments', 'iconwoo' ), "dependency" => $common_dependency );

		$default_tabs['icontact_woo_abn_cart'] = array( "name" => __( 'Abandoned Cart', 'iconwoo' ), "dependency" => $common_dependency );

		$default_tabs['icontact_woo_workflows'] = array( "name" => __( 'Workflows', 'iconwoo' ), "dependency" => $common_dependency );

		$default_tabs['icontact_woo_ocs'] = array( "name" => __( 'One Click Sync', 'iconwoo' ), "dependency" => $common_dependency );

		$default_tabs['icontact_woo_rfm_settings'] = array( "name" => __( 'RFM Settings', 'iconwoo' ), "dependency" => $common_dependency );

		$default_tabs['icontact_woo_settings'] = array( "name" => __( 'Settings', 'iconwoo' ), "dependency" => array("is_custom_field_setup_completed") );

		

		return $default_tabs;
	}

	/**
	 * checking dependencies for tabs
	 * @since     1.0.0
	 */
	public function check_dependencies( $dependency = array() ) {

		$flag = true;

		global $icontact_woo;

		if( count( $dependency ) ) {

			foreach( $dependency as $single_dependency ) {

				$flag &= $icontact_woo->$single_dependency();
			}
		}

		return $flag;
	}

	/**
	 * Locate and load appropriate tempate.
	 *
	 * @since 	1.0.0
	 */
	public function load_template_view( $path, $params=array() ){

		$file_path = ICONTACT_WOO_ABSPATH.$path;

		if( file_exists( $file_path ) ) {

			include $file_path;
		}
		else {

			$notice = sprintf( __( 'Unable to locate file path at location "%s". Some features may not work properly in iContact WooCommerce Integration, please contact us!', 'iconwoo' ) , $file_path );

			$this->icontact_woo_notice( $notice, 'error' );
		}
	}
	/**
	 * show admin notices.
	 * @param  string 	$message 	Message to display.
	 * @param  string 	$type    	notice type, accepted values - error/update/update-nag
	 * @since  1.0.0
	 */
	public static function icontact_woo_notice( $message, $type='error' ) {

		$classes = "notice ";
		
		switch($type){

			case 'update':
				$classes .= "updated";
				break;

			case 'update-nag':
				$classes .= "update-nag";
				break;
			case 'success':
				$classes .= "notice-success is-dismissible";
				break;

			default:
				$classes .= "error";
		} 

		$notice = '<div class="'. $classes .'">';
		$notice .= '<p>'. $message .'</p>';
		$notice .= '</div>';

		echo $notice;	
	}

	/**
	 * Retreving icontact app id
	 *
	 * @since 1.0.0
	 */
	public static function get_app_id(){

		$app_id = get_option('icontact_woo_app_id','');
		return $app_id;
	}

	/**
	 * Retreving icontact app password
	 *
	 * @since 1.0.0
	 */
	public static function get_app_password(){

		$app_password = get_option('icontact_woo_app_password','');
		return $app_password;
	}

	/**
	 * Retreving icontact API username
	 *
	 * @since 1.0.0
	 */
	public static function get_api_username(){

		$api_username = get_option('icontact_woo_api_username','');
		return $api_username;
	}

	/**
	 * Retreving icontact Company Id
	 *
	 * @since 1.0.0
	 */
	public static function get_company_id(){

		$company_id = get_option('icontact_woo_company_id','');
		return $company_id;
	}

	/**
	 * Retreving icontact profile Id
	 *
	 * @since 1.0.0
	 */
	public static function get_profile_id(){

		$profile_id = get_option('icontact_woo_profile_id','');
		return $profile_id;
	}

	/**
	 * checkng for get started call is executed or not
	 *
	 * @since 1.0.0
	 */
	public static function icontact_woo_get_started() {

		if( self::is_setup_completed() ) {

			return true;
		}
		else {

			return get_option( 'icontact_woo_get_started', false );
		}
	}

	/**
	 * verify if the icontact setup is completed.
	 *
	 * @since 1.0.0
	 */
	public static function is_custom_field_setup_completed(){

		return get_option('icontact_woo_setup_completed', false);
	}
	/**
	 * checking input fields for validation
	 *
	 * @since 1.0.0
	 */
	public static function is_valid_details_provided(){

		$status = '';
		$app_id = get_option('icontact_woo_app_id','');
		$app_password = get_option('icontact_woo_app_password','');
		$api_username = get_option('icontact_woo_api_username','');
		

		if (!$app_id) {
			$status = 'empty_app_id';
		}
		elseif (!$app_password) {
			$status = 'empty_app_password';
		}
		elseif (!$api_username) {
			$status = 'empty_api_username';
		}
		else{
			$status = 'ok';
		}

		return $status;
	}

	public static function is_auth_success(){

		return get_option('icontact_id_verified',false); 
	}

	/**
	 * check if plugin feature is enbled or not.
	 * @since  1.0.0 
	 * @return boolean true/false
	 */

	public function is_plugin_enable() {

		return get_option( "icontact_woo_settings_enable", "no" );
	}

	/**
	 * Check the user choice for fields setup
	 *
	 * @since 1.0.0
	 * @return yes/no
	 */

	public static function icontact_woo_user_choice() {

		return get_option( "icontact_woo_select_fields", "" );
	}

	/**
	 * verify if the icontact setup is completed.
	 *
	 * @since 1.0.0
	 */
	public static function is_setup_completed(){

		return get_option( 'icontact_woo_setup_completed', false );
	}

	/**
	 * returns the user selected fields for setup
	 *
	 * @since 1.0.0
	 * @return array of selected contact properties
	 */

	public static function icontact_woo_user_selected_fields() {

		return get_option( "icontact_woo_selected_properties", array() );
	}

	/**
	 * getting whether any fields selected to be created or not
	 *
	 * @since 1.0.0
	 */

	public static function is_fields_to_create() {

		$choice = self::icontact_woo_user_choice();
		$status = false;

		if( $choice == 'no' ) {

			$status = true;
		}
		elseif( $choice == 'yes' ) {

			$selected_properties = self::icontact_woo_user_selected_fields();

			if( is_array( $selected_properties ) && count( $selected_properties ) ) {

				$status = true;
			}
		}

		return $status;
	}

	/**
	 * getting final custom fields 
	 *
	 * @since 1.0.0
	 */

	public static function icontact_woo_get_final_fields() {

		$final_properties = array();

		$groups = IcontactWooContactProperties::get_instance()->_get( 'groups' );

		$existing_fields = get_option( "icontact-woo-fields-created", array() );

		if( self::icontact_woo_user_choice() == 'no' ) {

			if( is_array( $groups ) && count( $groups ) ) {

				foreach( $groups as $single_group ) {

					$properties = IcontactWooContactProperties::get_instance()->_get( 'properties', $single_group["name"] );

					if( is_array( $properties ) && count( $properties ) ) {

						foreach( $properties as $single_property ) {

							if( in_array( $single_property['alias'], $existing_fields ) ) {

								$final_properties[] = array( 'detail' => $single_property, 'status' => 'created' );
							}
							else {

								$final_properties[] = array( 'detail' => $single_property, 'status' => 'false' );
							}
						}
					}
				}
			}
		}
		else {

			$selected_properties = self::icontact_woo_user_selected_fields();

			if( is_array( $groups ) && count( $groups ) ) {

				foreach( $groups as $single_group ) {

					$properties = IcontactWooContactProperties::get_instance()->_get( 'properties', $single_group["name"] );

					if( is_array( $properties ) && count( $properties ) ) {

						foreach( $properties as $single_property ) {

							if( in_array( $single_property["alias"], $selected_properties ) ) {

								if( in_array( $single_property['alias'], $existing_fields ) ) { 

									$final_properties[] = array( 'detail' => $single_property, 'status' => 'created' );
								}
								else {

									$final_properties[] = array( 'detail' => $single_property, 'status' => 'false' );
								}
							}
							else {

								$final_properties[] = array( 'detail' => $single_property, 'status' => 'false' );
							}
						}
					}
				}
			}
		}

		return $final_properties;
	}
}



