<?php 
/**
 * All property callbacks.
 *
 * @link       https://makewebbetter.com/
 * @since      1.0.0
 *
 * @package    iconwoo-integration
 * @subpackage iconwoo-integration/includes
 */

/**
 * Manage all property callbacks.
 *
 * Provide a list of functions to manage all the information
 * about contacts properties and there callback functions to
 * get value of that property.
 *
 * @package    iconwoo-integration
 * @subpackage iconwoo-integration/includes
 * @author     MakeWebBetter <webmaster@makewebbetter.com>
 */

class IcontactWooPropertyCallbacks{

	/**
	 * contact id.
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var int
	 */
	protected $_contact_id;

	/**
	 * WP user.
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var WP_User
	 */
	protected $_user;

	/**
	 * cache values.
	 * @var array.
	 */
	protected $_cache = array();

	/**
	 * properties and there callbacks.
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var Associated_array
	 */
	protected $_property_callbacks = array(

		'mwb_customer_group'		=> 'get_contact_group',
		'mwb_newsletter_subs' 		=> 'iconwoo_user_meta',
		'mwb_customer_cart_id' 		=> 'iconwoo_user_meta',

		'mwb_last_order_stat' 		=> 'iconwoo_user_meta',
		'mwb_last_order_ff_stat' 	=> 'iconwoo_user_meta',
		'mwb_last_order_track_num' 	=> 'iconwoo_user_meta',
		'mwb_last_order_ship_date' 	=> 'iconwoo_user_meta',
		'mwb_last_order_num' 		=> 'iconwoo_user_meta',
		'mwb_current_orders' 		=> 'iconwoo_user_meta',

		'mwb_total_val_of_orders' 	=> 'iconwoo_user_meta',
		'mwb_avg_order_value'	 	=> 'iconwoo_user_meta',
		'mwb_total_orders' 			=> 'iconwoo_user_meta',
		'mwb_first_order_date' 		=> 'iconwoo_user_meta',
		'mwb_first_order_val' 		=> 'iconwoo_user_meta',
		'mwb_last_order_date' 		=> 'iconwoo_user_meta',
		'mwb_last_order_val' 		=> 'iconwoo_user_meta',
		'mwb_avg_days_bt_orders' 	=> 'iconwoo_user_meta',
		'mwb_acc_creation_date' 	=> 'iconwoo_user_meta',
		'mwb_order_monetary' 		=> 'iconwoo_user_meta',
		'mwb_order_frequency' 		=> 'iconwoo_user_meta',
		'mwb_order_recency' 		=> 'iconwoo_user_meta',

		'mwb_lifecycle_stage'		=> 'iconwoo_user_meta',
		);

	/**
	 * constructor
	 * @param int    $contact_id    contact id to get property values of.
	 */

	public function __construct($contact_id){

		$this->_contact_id = $contact_id;
		$this->_user = get_user_by('id' ,$this->_contact_id);
	}

	/**
	 * property value.
	 *
	 * @param  string  $property_name    name of the contact property.
	 * @since 1.0.0
	 */
	public function  _get_property_value($property_name){

		$value = "" ;
		if (!empty($property_name)) {
			
			//get the callback
			$callback_function = $this->_get_property_callback($property_name);

			if (!empty($callback_function)) {
				
				// get the value by calling respective callback.
				$value = $this->$callback_function($property_name);
			}
		}

		$value = apply_filters( 'icontact_woo_contact_property_value', $value, $property_name, $this->_contact_id );

		return $value;
	}

	/**
	 * filter the property callback to get value of.
	 * 
	 * @param  strig   $property_name   name of the property.
	 * @return string/false             callback function name or false.
	 */
	public function _get_property_callback($property_name){

		//check if property name exists in the array.
		if (array_key_exists($property_name, $this->_property_callbacks)) {
			$callback = $this->_property_callbacks[ $property_name ];

			return $callback;
		}

		return false;
	}

	/**
	 * get contact user role.
	 *
	 * @return string    user role of the current contact.
	 * @since 1.0.0
	 */
	public function get_contact_group(){

		//get roles from user object.
		$user_roles = isset( $this->_user->roles) ? ($this->_user->roles) : "" ;

		return $user_roles;
	}
	
	/**
	 * user email
	 * @since 1.0.0
	 */
	public function _get_email(){

		return $this->_user->data->user_email;
	}

	/**
	 * user details with iconwoo_ prefix.
	 * 
	 * @since 1.0.0
	 */
	public function iconwoo_user_meta($key){

		//check if property value is already in cache.
		if (array_key_exists($key, $this->_cache)) {
			//return the cache value.
			return $this->_cache[$key];
		}

		$selected_order_statuses = get_option("icontact-woo-selected-order-status", array() );

		if (empty($selected_order_statuses)) {
			
			$selected_order_statuses = array_keys( wc_get_order_statuses() );
		}

		//get all the orders of the contact.
		$customer_orders = get_posts(array(
			'numberposts' 	=> -1,
			'meta_key'		=> '_customer_user',
			'meta_value'	=> $this->_contact_id,
			'post_type'		=> wc_get_order_types(),
			'post_status'	=> array_keys( wc_get_order_statuses() ),
			'order'			=> 'DESC',
		));

		$customer = new WP_User($this->_contact_id);
		$order_frequency 	= 0;
		$account_creation 	= isset($customer->data->user_registered) ? $customer->data->user_registered : "" ;
		$account_creation 	= strtotime($account_creation);

		$this->_cache['mwb_acc_creation_date']	= date( 'Y/m/d', $account_creation);
		$this->_cache['mwb_current_orders'] 	= 0;
		$this->_cache['mwb_order_recency'] 		= 1;
		$this->_cache['mwb_order_frequency'] 	= 1;
		$this->_cache['mwb_order_monetary'] 	= 1;

		$contact_newsletter_subscription = 'no';
		
		$contact_newsletter_subscription = apply_filters( "icontact_woo_user_newsletter_subscription", $contact_newsletter_subscription, $this->_contact_id );
		$this->_cache['mwb_newsletter_subs'] = $contact_newsletter_subscription;
		$this->_cache['mwb_total_val_of_orders'] = 0;

		//deciding lifecyle stage of customer
		if (count($customer_orders) < 1) {

			$this->_cache['mwb_lifecycle_stage'] = 'Marketing Qualified Lead';

		}
		else{
			$this->_cache['mwb_lifecycle_stage'] = 'Customer';

		}
		
		//if customer have orders
		if (is_array($customer_orders) && count($customer_orders)) {
			
			//total number of customer orders.
			$this->_cache['mwb_total_orders'] = count($customer_orders);
			$order_frequency = $this->_cache['mwb_total_orders'];
			$counter = 0;
			$products_count = 0;

			foreach ($customer_orders as $order_details) {
				
				//get the order id
				$order_id = isset($order_details->ID) ? intval($order_details->ID) : 0 ;

				//if order id is not found check for another order
				if (!$order_id) {
					continue;
				}

				//order date
				$order_date = get_post_time('U', true, $order_id);

				//get_order
				$order = new WC_Order($order_id);

				//check for wp_error object
				if(empty($order) || is_wp_error($order))
					continue;

				$order_status = $order->get_status();
				$order_total = $order->get_total();

				$this->_cache['mwb_total_val_of_orders'] += floatval( $order_total );
				
				if ($order_status !== 'failed' && $order_status !== 'cancelled' && $order_status !== 'refunded' && $order_status !== 'completed')
				{
						$this->_cache['mwb_current_orders'] += 1;
				}

				//check for last order and finish all last order calculations.
				if (!$counter) {
					
					//last order date
					$this->_cache['mwb_last_order_date'] = date('Y/m/d', get_post_time('U', true, $order_id));
					$last_order_date = get_post_time('U', true, $order_id);
					$this->_cache['mwb_last_order_val'] = $order_total;
					$this->_cache[ 'mwb_last_order_num' ] = $order_id;
					$this->_cache[ 'mwb_last_order_ff_stat' ] = "wc-".$order->get_status();						
					$this->_cache[ 'mwb_last_order_ship_date' ] = apply_filters( 'iconwoo_order_shipment_date', '', $order_id );
					$this->_cache[ 'mwb_last_order_track_num' ] = $order->get_order_number();
					$this->_cache[ 'mwb_last_order_stat' ] = "wc-".$order->get_status();
				}

				// check for first order.
				if( $counter == ( count( $customer_orders ) - 1 ) )
				{
					// first order based calculation here..
					$this->_cache[ 'mwb_first_order_date' ] = date( 'Y/m/d', get_post_time( 'U', true, $order_id ) );
					$this->_cache[ 'mwb_first_order_val' ] = $order_total;
				}

				$counter++;
			}
			$this->_cache['mwb_total_val_of_orders'] = round($this->_cache['mwb_total_val_of_orders'], 2);
			// rest calculations here.
			$this->_cache[ 'mwb_avg_order_value' ] = floatval( $this->_cache[ 'mwb_total_val_of_orders' ] / $this->_cache[ 'mwb_total_orders' ] );
			$this->_cache[ 'mwb_avg_order_value' ] = round($this->_cache[ 'mwb_avg_order_value' ], 2);
			$iconwoo_rfm_at_5    = array( 0 => 30, 1 => 20, 2 => 1000 );
	        $iconwoo_from_rfm_4  = array( 0 => 31, 1 => 10, 2 => 750 );
	        $iconwoo_to_rfm_4    = array( 0 => 90, 1 => 20, 2 => 1000 );
	        $iconwoo_from_rfm_3  = array( 0 => 91, 1 => 5, 2 => 500 );
	        $iconwoo_to_rfm_3    = array( 0 => 180, 1 => 10, 2 => 750 );
	        $iconwoo_from_rfm_2  = array( 0 => 181, 1 => 2, 2 => 250 );
	        $iconwoo_to_rfm_2    = array( 0 => 365, 1 => 5, 2 => 500 );
	        $iconwoo_rfm_at_1    = array( 0 => 365, 1 => 2, 2 => 250 );

			$order_monetary = $this->_cache[ 'mwb_total_val_of_orders' ];

			$current_date 		= gmdate("Y-m-d H:i:s", time() );
			$current_date 		= new DateTime($current_date);
			$last_order_date 	= gmdate("Y-m-d H:i:s", $last_order_date);
			$last_order_date 	= new DateTime($last_order_date);
			$order_recency 		= date_diff( $current_date, $last_order_date, true );
			
			$order_recency = $order_recency->days;

			if( $order_recency <= $iconwoo_rfm_at_5[0] )
			{
				$this->_cache[ 'mwb_order_recency' ] = 5;
			}
			elseif( $order_recency >= $iconwoo_from_rfm_4[0] && $order_recency <= $iconwoo_to_rfm_4[0] )
			{
				$this->_cache[ 'mwb_order_recency' ] = 4;
			}
			elseif($order_recency >= $iconwoo_from_rfm_3[0] && $order_recency <= $iconwoo_to_rfm_3[0] )
			{
				$this->_cache[ 'mwb_order_recency' ] = 3;
			}
			elseif( $order_recency >= $iconwoo_from_rfm_2[0] && $order_recency <= $iconwoo_to_rfm_2[0] )
			{
				$this->_cache[ 'mwb_order_recency' ] = 2;
			}
			elseif( $order_recency > $iconwoo_rfm_at_1[0] )
			{
				$this->_cache[ 'mwb_order_recency' ] = 1;
			}

			if( $order_frequency >= $iconwoo_rfm_at_5[1] )
			{
				$this->_cache[ 'mwb_order_frequency' ] = 5;
			}
			elseif( $order_frequency >= $iconwoo_from_rfm_4[1] && $order_frequency < $iconwoo_to_rfm_4[1] )
			{
				$this->_cache[ 'mwb_order_frequency' ] = 4;
			}
			elseif( $order_frequency >= $iconwoo_from_rfm_3[1] && $order_frequency < $iconwoo_to_rfm_3[1] )
			{
				$this->_cache[ 'mwb_order_frequency' ] = 3;
			}
			elseif( $order_frequency >= $iconwoo_from_rfm_2[1] && $order_frequency < $iconwoo_to_rfm_2[1] )
			{
				$this->_cache[ 'mwb_order_frequency' ] = 2;
			}
			elseif( $order_frequency < $iconwoo_rfm_at_1[1] )
			{
				$this->_cache[ 'mwb_order_frequency' ] = 1;
			}

			if( $order_monetary >= $iconwoo_rfm_at_5[2] )
			{
				$this->_cache[ 'mwb_order_monetary' ] = 5;
			}
			elseif( $order_monetary >= $iconwoo_from_rfm_4[2] && $order_monetary < $iconwoo_to_rfm_4[2] )
			{
				$this->_cache[ 'mwb_order_monetary' ] = 4;
			}
			elseif( $order_monetary >= $iconwoo_from_rfm_3[2] && $order_monetary < $iconwoo_to_rfm_3[2] )
			{
				$this->_cache[ 'mwb_order_monetary' ] = 3;
			}
			elseif( $order_monetary >= $iconwoo_from_rfm_2[2] && $order_monetary < $iconwoo_to_rfm_2[2] )
			{
				$this->_cache[ 'mwb_order_monetary' ] = 2;
			}
			elseif( $order_monetary < $iconwoo_rfm_at_1[2] )
			{
				$this->_cache[ 'mwb_order_monetary' ] = 1;
			}
		}

		if( isset( $this->_cache[ $key ] ) ) {

			return $this->_cache[ $key ];
		}
	}
}

?>